/*
 * gimp-image-reg -- A GIMP Plugin for Image Registration
 *
 * image-ops.h
 * Copyright (C) 2010 Behnam Tabatabai
 *
 * This file is part of gimp-image-reg.
 *
 * gimp-image-reg is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * gimp-image-reg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gimp-image-reg. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __image_ops_h
#define __image_ops_h

#include "matrix.h"
#include <libgimp/gimp.h>
#include <glib.h>

typedef struct {
    gint32        x_offset;
    gint32        y_offset;
    gint32        width;
    gint32        height;
    gint32        x_roi;
    gint32        y_roi;
    gint32        w_roi;
    gint32        h_roi;
    Matrix_Uint8* pixel;
    Matrix_Uint8* mask;
    Matrix_Int16* gradient;
    Matrix_Real*  steepest_descent;
    Matrix_Real*  hessian;
} Image;

void ReleaseImage(Image* image);

Matrix_Uint8* Extract_GrayScaleImage(
    GimpDrawable* drawable,
    gint32 x,
    gint32 y,
    gint32 width,
    gint32 height);

Matrix_Int16* CreateGradient(const Matrix_Uint8* image);

/* Experimental. */
gboolean Image_WritePgm(
    const gchar* name,
    const Matrix_Uint8* image);

gboolean Gradient_WritePgm(
    const gchar* name,
    const Matrix_Int16* image,
    gint channel);

#endif
