/*
 * gimp-image-reg -- A GIMP Plugin for Image Registration
 *
 * matrix.h
 * Copyright (C) 2010 Behnam Tabatabai
 *
 * This file is part of gimp-image-reg.
 *
 * gimp-image-reg is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * gimp-image-reg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gimp-image-reg. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __matrix_h
#define __matrix_h

#include <glib.h>

/* Column-major access to matrix elements. */
#define NUM_ROWS(matrix) ((matrix)->num_rows)
#define NUM_COLS(matrix) ((matrix)->num_cols)
#define NUM_CHNS(matrix) ((matrix)->num_chns)

#define ELEM(matrix, row, col, channel)  \
    ((matrix)->elements +                \
     NUM_CHNS(matrix) *                  \
     (NUM_ROWS(matrix) * (col) +         \
      (row)) +                           \
     (channel))

typedef struct {
    gint32 num_rows;
    gint32 num_cols;
    gint32 num_chns;
    guint8* elements;
} Matrix_Uint8;

typedef struct {
    gint32 num_rows;
    gint32 num_cols;
    gint32 num_chns;
    gint16* elements;
} Matrix_Int16;

typedef struct {
    gint32 num_rows;
    gint32 num_cols;
    gint32 num_chns;
    gdouble* elements;
} Matrix_Real;


Matrix_Uint8* CreateMatrix_Uint8(
    gint32 num_rows,
    gint32 num_cols,
    gint32 num_chns);

Matrix_Int16* CreateMatrix_Int16(
    gint32 num_rows,
    gint32 num_cols,
    gint32 num_chns);

Matrix_Real* CreateMatrix_Real(
    gint32 num_rows,
    gint32 num_cols,
    gint32 num_chns);

void DeleteMatrix_Uint8(Matrix_Uint8* matrix);
void DeleteMatrix_Int16(Matrix_Int16* matrix);
void DeleteMatrix_Real(Matrix_Real* matrix);

void MatrixSetIdentity(Matrix_Real* matrix);
void MatrixSetZero(Matrix_Real* matrix);

void MatrixMultiply(
    const Matrix_Real* a,
    const Matrix_Real* b,
    Matrix_Real* c);

#endif
