/*
 * gimp-image-reg -- A GIMP Plugin for Image Registration
 *
 * user-data.h
 * Copyright (C) 2010 Behnam Tabatabai
 *
 * This file is part of gimp-image-reg.
 *
 * gimp-image-reg is free software: you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * gimp-image-reg is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
 * General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with gimp-image-reg. If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef __user_data_h
#define __user_data_h

#include <glib.h>
#include <libgimp/gimp.h>

typedef enum {
    MATCHING_AREA_WHOLE_IMAGE,
    MATCHING_AREA_SELECTION,
} MatchingArea;

typedef enum {
    XFORMATION_RIGID,
    XFORMATION_AFFINE,
} TransformationType;

typedef struct {
    /* Reference drawable. */
    gint32 ref_drawable;
    
    /* Matching area. */
    MatchingArea matching_area;
    
    /* Transformation type. */
    TransformationType transformation;

    /* Unlimited number of iterations? */
    gboolean unlimited_iterations;
    
    /* The maximum number of iterations, if limited. */
    gint32 num_iterations;

    /* Accuracy is given as n -> 10^(-n) */
    gint32 accuracy;

    /* Type of interpolation for the transformed image. */
    GimpInterpolationType interpolation;
    
    /* How to clip results. */
    GimpTransformResize clip_method;

} UserData;


void UserData_GetDefaults(UserData*);

#endif
