/* GIMP Plug-in Template
 * Copyright (C) 2000  Michael Natterer <mitch@gimp.org> (the "Author").
 * All Rights Reserved.
 *
 * Permission is hereby granted, free of charge, to any person obtaining a
 * copy of this software and associated documentation files (the "Software"),
 * to deal in the Software without restriction, including without limitation
 * the rights to use, copy, modify, merge, publish, distribute, sublicense,
 * and/or sell copies of the Software, and to permit persons to whom the
 * Software is furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included
 * in all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
 * OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
 * THE AUTHOR BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
 * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
 * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 *
 * Except as contained in this notice, the name of the Author of the
 * Software shall not be used in advertising or otherwise to promote the
 * sale, use or other dealings in this Software without prior written
 * authorization from the Author.
 */

#include "config.h"

#include <libgimp/gimp.h>
#include <libgimp/gimpui.h>

#include "main.h"
#include "interface.h"

#include "plugin-intl.h"


/*  Constants  */

#define SCALE_WIDTH        180
#define SPIN_BUTTON_WIDTH   75
#define RANDOM_SEED_WIDTH  100


/*  Local function prototypes  */

static void       dialog_toggle_callback            (GtkWidget *widget,
						     gpointer   data);


/*  Local variables  */

static PlugInUIVals *ui_state = NULL;

static GtkWidget *hor = NULL;
static GtkWidget *ver = NULL;

/*  Public functions  */

gboolean
dialog (gint32              image_ID,
	GimpDrawable       *drawable,
	PlugInVals         *vals,
	PlugInImageVals    *image_vals,
	PlugInDrawableVals *drawable_vals,
	PlugInUIVals       *ui_vals)
{
  GtkWidget *dlg;
  GtkWidget *main_vbox;
  GtkWidget *label;

  gboolean   run = FALSE;
  ui_state = ui_vals;

  gimp_ui_init (PLUGIN_NAME, TRUE);

  dlg = gimp_dialog_new (_("Intelligum"), PLUGIN_NAME,
                         NULL, 0,
			 gimp_standard_help_func, "plug-in-intelligum",

			 GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL,
			 GTK_STOCK_OK,     GTK_RESPONSE_OK,

			 NULL);

  main_vbox = gtk_vbox_new (FALSE, 4);
  gtk_container_set_border_width (GTK_CONTAINER (main_vbox), 6);
  gtk_container_add (GTK_CONTAINER (GTK_DIALOG (dlg)->vbox), main_vbox);


  label = gtk_label_new(_("Erase selection, and fill with interpollated background"));
  gtk_box_pack_start (GTK_BOX (main_vbox), label, TRUE, FALSE, 0);
  gtk_widget_show (label);

  hor = gtk_check_button_new_with_mnemonic(_("horizontal"));
  gtk_box_pack_start (GTK_BOX (main_vbox), hor, TRUE, FALSE, 0);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (hor), vals->do_hor);
  gtk_widget_show (hor);

  ver = gtk_check_button_new_with_mnemonic(_("vertical"));
  gtk_box_pack_start (GTK_BOX (main_vbox), ver, TRUE, FALSE, 0);
  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ver), vals->do_ver);
  gtk_widget_show (ver);

  g_signal_connect (hor, "toggled",
                    G_CALLBACK (dialog_toggle_callback),
                    &vals->do_hor);
  g_signal_connect (ver, "toggled",
                    G_CALLBACK (dialog_toggle_callback),
                    &vals->do_ver);

  /*  Show the main containers  */

  gtk_widget_show (main_vbox);
  gtk_widget_show (dlg);

  run = (gimp_dialog_run (GIMP_DIALOG (dlg)) == GTK_RESPONSE_OK);

  gtk_widget_destroy (dlg);

  return run;
}


/*  Private functions  */

static void
dialog_toggle_callback (GtkWidget *widget,
                    gpointer   data)
{
  gint val = GTK_TOGGLE_BUTTON (widget)->active;
  if (val)
    *(gint *) data = TRUE;
  else
    *(gint *) data = FALSE;
  if (!val) {
    if (widget == hor) {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (ver), TRUE);
    } else {
      gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON (hor), TRUE);
    }
  }
}

